%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% This is the varification of DM2 proposed in the following paper:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Hong Jia, Yiu-Ming Cheung, Jiming Liu, 
%%%%%% A new distance metric for unsupervised learning of categorical data, 
%%%%%% IEEE Transactions on Neural Networks and Learning Systems, 2016, 27(5): 1065~1079   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% Please cite the paper if you use this code.

close all;
clear all;

%% Input the data set

%
xx = textread('./dataset/new_vote.txt');  %%% input data sets
%%%435 instances, 267 for class 1 and 168 for class 2. have 16 features.
%%%totally have 3 different categorical value, first column is label.
x = xx(:,2:17);
true_class = xx(:,1);
class_label = true_class';
k = 2;  %%% number of clusters
m = 3; %%% the maximum number of different values can be chosen by each attribute  
%}


%%
[n, d] = size(x);  %%% n: the number of samples;   d: the number of attributes(dimensionality)
number_in_class = zeros(1, k);
for i = 1:n
    number_in_class(class_label(i)) = number_in_class(class_label(i)) + 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
frequency_in_data = zeros(d, m);  %%%%count the frequency of each attribute value amoug the whole dataset
for i = 1:n
    for j = 1:d
        frequency_in_data(j,x(i,j)) = frequency_in_data(j,x(i,j))+1;
    end
end
proba_in_data = frequency_in_data./n;
    
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Used to calculate dynamic weight
equal_proba = zeros(1, d);
for i = 1:d
    sum_prob = 0;
    for j = 1:m
        sum_prob = sum_prob +  (proba_in_data(i, j))*((frequency_in_data(i, j)-1)/(n-1));
    end
    equal_proba(i) = sum_prob;
end
differ_proba = 1 - equal_proba;

%%    
%%%%%%%%%%%%%%%%%%%%%%%% Calculate the inter- and intra- cluster distances
objects_distance_matrix = zeros(k, k);
max_dis = 0;
for i = 1:n
    for j = 1:n
        dis = 0;
        sum_weight = 0;
        for h = 1:d
            if x(i, h) ~= x(j, h)
                dis = dis + equal_proba(h)*((proba_in_data(h, x(i, h)))*((frequency_in_data(h, x(i, h))-1)/(n-1)) +(proba_in_data(h, x(j, h)))*((frequency_in_data(h, x(j, h))-1)/(n-1)));
                sum_weight = sum_weight + equal_proba(h);
            else
                sum_weight = sum_weight + differ_proba(h);
            end
        end
        object_dis = dis/sum_weight;

        if object_dis > max_dis
            max_dis = object_dis;
        end
        objects_distance_matrix(class_label(i), class_label(j)) = objects_distance_matrix(class_label(i), class_label(j)) + object_dis;
    end
end
    
for i = 1:k
    for j = 1:k
        number_dis = number_in_class(i) * number_in_class(j);
        objects_distance_matrix(i, j) = objects_distance_matrix(i, j)/(number_dis * max_dis);
    end
end
                    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
index_x = 0;
for i = 1:k
    v1 = objects_distance_matrix(i, i);
    v2 = sum(objects_distance_matrix(i, :));
    v2 = v2 - v1;
    v3 = v2/(k-1);
    index_x = index_x + v1/v3;
end
index_x = index_x/k;

objects_distance_matrix
index_x

